//
//  SQTDDocument.swift
//  SQToDo
//
//  Created by narita on 2018/05/06.
//  Copyright © 2018年 narita. All rights reserved.
//

import Cocoa

class SQTDDocument: NSDocument
{
    // MARK: - class methods
    
    override class var autosavesInPlace: Bool
    {
        return true
    }

    // MARK: - init methods

    override init()
    {
        super.init()
        // Add your subclass-specific initialization here.
    }

    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // UTIを決定する
    
    override func fileWrapper(ofType typeName: String) throws -> FileWrapper
    {
        // ファイル形式名をUTIは"com.mindTo01s.macOS.sqtdDoc"
        if !(NSWorkspace.shared.type("com.mindTo01s.macOS.sqtdDoc" as String, conformsToType: typeName))
        {
            NSException(
                name: NSExceptionName.invalidArgumentException,
                reason: "\(typeName) is not a recognized document type.").raise()
        }
        
        return FileWrapper(regularFileWithContents:NSKeyedArchiver.archivedData(withRootObject: self.model))
    }
   
    override func read(from fileWrapper: FileWrapper, ofType typeName: String) throws
    {
        // "com.mindTo01s.macOS.sqtdDoc" 以外はサポートしない
        if !(NSWorkspace.shared.type("com.mindTo01s.macOS.sqtdDoc" as String, conformsToType: typeName))
        {
            NSException(
                name: NSExceptionName.invalidArgumentException,
                reason: "\(typeName) is not a recognized document type.").raise()
        }

        // バンドル形式(ディレクトリ形式)はサポートしない
        if !fileWrapper.isRegularFile
        {
            NSException(
                name: NSExceptionName.objectNotAvailableException,
                reason: "fileWrapper is noy RegularFile.").raise()
        }

        self.model = NSKeyedUnarchiver.unarchiveObject(with: fileWrapper.regularFileContents!) as! SQTDModel
    }

    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @objc var model: SQTDModel = SQTDModel()
    
    // MARK: - Utility methods

    override func makeWindowControllers()
    {
        // Returns the Storyboard that contains your Document window.
        let storyboard = NSStoryboard(name: NSStoryboard.Name("Main"), bundle: nil)
        let windowController = storyboard.instantiateController(withIdentifier: NSStoryboard.SceneIdentifier("Document Window Controller")) as! NSWindowController
        self.addWindowController(windowController)
    }

}

